<?php
/**
 * utility na odesilani e-mailu
 *
 * @package sendmail
 * @author fish <fish@archa.cz>
 */

  // {{{ guessMIMEType
  /**
   * odhadne MIME typ souboru podle koncovky
   *
   * @param string $filename
   * @return string
   */
  function guessMIMEType($filename){
    $filename=preg_replace("/^([^\/]+\/)+/","",$filename);

    if(strrchr($filename,".") == false) {
      return("application/octet-stream");
    }

    $ext = strrchr($filename,".");
    switch($ext) {
      case ".gif":
        return "image/gif";
        break;
      case ".png":
        return "image/png";
        break;
      case ".gz":
        return "application/x-gzip";
      case ".htm":
      case ".html":
        return "text/html";
        break;
      case ".jpg":
      case ".jpeg":
        return "image/jpeg";
        break;
      case ".tar":
        return "application/x-tar";
        break;
      case ".txt":
        return "text/plain";
        break;
      case ".zip":
        return "application/zip";
        break;
      default:
        return "application/octet-stream";
        break;
    }
  }
  // }}} guessMIMEType


  // {{{ sendmail
  /**
   * odesle plaintext e-mail na zadanou adresu
   *
   * funkce pocita s kodovanim utf-8
   *
   * @param string $mfrom odesilatel
   * @param string $mto adresat
   * @param string $msub predmet
   * @param string $mmsg plaintext obsah zpravy
   * @return boolean
   */
  function sendmail($mfrom, $mto, $msub, $mmsg, $replyto=null){
    $msub=preg_replace("/[\n\r]+/"," ",$msub);
    $mto=preg_replace("/[\n\r]+/"," ",$mto);
    $mfrom=preg_replace("/[\n\r]+/"," ",$mfrom);
    if (empty($replyto)) {
      $replyto = $mfrom;
    }
    $mailsub="=?utf-8?B?".base64_encode($msub)."?=";
    $mailmsg=$mmsg;
    $mailfrom=$mfrom;
    $mailfrom2=$mfrom;  //samotny mail bez komentaru !
    $mailhead="From: ".$mailfrom."\nReply-To: ".$replyto."\nContent-Type: text/plain;\n\tcharset=\"utf-8\"\nContent-Transfer-Encoding: 8bit\n";
    return(mail($mto,$mailsub,$mailmsg,$mailhead,"-B8BITMIME -f ".$mailfrom2));
  }
  // }}} sendmail


  // {{{ sendmail2
  /**
   * odesle plaintext/HTML e-mail na zadanou adresu
   *
   * funkce pocita s kodovanim utf-8
   *
   * @param string $mfrom odesilatel
   * @param string $mto adresat
   * @param string $msub predmet
   * @param string $mmsg plaintext obsah zpravy
   * @param string $mmsg2 HTML obsah zpravy
   * @return boolean
   */
  function sendmail2($mfrom,$mto,$msub,$mmsg,$mmsg2 ){
    $msub=preg_replace("/[\n\r]+/"," ",$msub);
    $mto=preg_replace("/[\n\r]+/"," ",$mto);
    $mfrom=preg_replace("/[\n\r]+/"," ",$mfrom);

    $mailsub="=?utf-8?B?".base64_encode($msub)."?=";
    $mailmsg=$mmsg;
    $mailfrom=$mfrom;
    $mailfrom2=$mfrom;  //samotny mail bez komentaru !
    $boundary="----mime-multipart-mail-".date("YmdHis");

    $mailhead="";
    $mailhead.="MIME-Version: 1.0\nContent-Type: multipart/alternative; boundary=\"".$boundary."\"\n";
    $mailhead.="From: ".$mailfrom."\nReply-To: ".$mailfrom."\n";

    $mailmsg_res="";
    $mailmsg_res.="This is a multi-part message in MIME format.\n\n";

    $mailmsg_res.="--".$boundary."\n";
    $mailmsg_res.="Content-Type: text/plain;\n";
    $mailmsg_res.="\tcharset=\"utf-8\"\n";
    $mailmsg_res.="Content-Transfer-Encoding: 8bit\n\n";
    $mailmsg_res.=$mmsg."\n";

    $mailmsg_res.="--".$boundary."\n";
    $mailmsg_res.="Content-Type: text/html;\n";
    $mailmsg_res.="\tcharset=\"utf-8\"\n";
    $mailmsg_res.="Content-Transfer-Encoding: 8bit\n\n";
    $mailmsg_res.=$mmsg2."\n";
    $mailmsg_res.="--".$boundary."--\n";

    return(mail($mto,$mailsub,$mailmsg_res,$mailhead,"-B8BITMIME -f ".$mailfrom2));
  }
  // }}} sendmail2


  // {{{ sendmail3
  /**
   * odesle plaintext/HTML e-mail na zadanou adresu s uvedenymi prilohami
   *
   * - funkce pocita s kodovanim utf-8
   * - lze pouzit libovolnou kombinaci priloh
   * - pole priloh musi obsahovat absolutni cesty k souborum priloh
   * - funkce zavisi na utilite mimencode a zakodovane prilohy si uklada do cache v /tmp
   *
   * @param string $mfrom odesilatel
   * @param string $mto adresat
   * @param string $msub predmet
   * @param string $mmsg plaintext obsah zpravy
   * @param string $mmsg2 HTML obsah zpravy
   * @param array $imgatt prilohy (obrazky), ktere budou pouzity v HTML verzi zpravy
   * @param array $imgatt_ext prilohy, ktere budou v mailu vlozene jako soubory
   * @return boolean
   */
  function sendmail3($mfrom,$mto,$msub,$mmsg,$mmsg2,$imgatt=array(),$imgatt_ext=array()){
    $msub=preg_replace("/[\n\r]+/"," ",$msub);
    $mto=preg_replace("/[\n\r]+/"," ",$mto);
    $mfrom=preg_replace("/[\n\r]+/"," ",$mfrom);

    $mailsub="=?utf-8?B?".base64_encode($msub)."?=";
    $mailmsg=$mmsg;
    $mailfrom=$mfrom;
    $mailfrom2=$mfrom;  //samotny mail bez komentaru !
    $boundary="----mime-multipart-mail-".date("YmdHis");
    $boundary_parts="----mime-multipart-mail-".date("YmdHis")."_parts_ext";
    $boundary_parts2="----mime-multipart-mail-".date("YmdHis")."_parts";

    if(count($imgatt) && count($imgatt_ext)==0){
      $mailhead.="MIME-Version: 1.0\nContent-Type: multipart/related; boundary=\"".$boundary."\"\n";
    } else {
      $mailhead.="MIME-Version: 1.0\nContent-Type: multipart/mixed; boundary=\"".$boundary."\"\n";
    }
    $mailhead.="From: ".$mailfrom."\nReply-To: ".$mailfrom."\n";

    $mailmsg_res.="This is a multi-part message in MIME format.\n\n";

    if(count($imgatt) && count($imgatt_ext)==0){
      $mailmsg_res.="--".$boundary."\n";
      $mailmsg_res.="Content-Type: multipart/alternative;\n";
      $mailmsg_res.="\tboundary=\"".$boundary_parts."\"\n\n";
    } else {
      if(count($imgatt)){
        $mailmsg_res.="--".$boundary."\n";
        $mailmsg_res.="Content-Type: multipart/related;\n";
        $mailmsg_res.="\ttype=\"multipart/alternative\";\n";
        $mailmsg_res.="\tboundary=\"".$boundary_parts."\"\n\n";
      }

      $mailmsg_res.="--".$boundary."\n";
      $mailmsg_res.="Content-Type: multipart/alternative;\n";
      $mailmsg_res.="\tboundary=\"".$boundary_parts2."\"\n\n";
    }

    if(count($imgatt) && count($imgatt_ext)==0){
      $mailmsg_res.="\n--".$boundary_parts."\n";
    } else {
      $mailmsg_res.="\n--".$boundary_parts2."\n";
    }
    $mailmsg_res.="Content-Type: text/plain;\n";
    $mailmsg_res.="\tcharset=\"utf-8\"\n";
    $mailmsg_res.="Content-Transfer-Encoding: 8bit\n\n";
    $mailmsg_res.=$mmsg."\n";

    if(count($imgatt) && count($imgatt_ext)==0){
      $mailmsg_res.="\n--".$boundary_parts."\n";
    } else {
      $mailmsg_res.="\n--".$boundary_parts2."\n";
    }
    $mailmsg_res.="Content-Type: text/html;\n";
    $mailmsg_res.="\tcharset=\"utf-8\"\n";
    $mailmsg_res.="Content-Transfer-Encoding: 8bit\n\n";
    $mailmsg_res.=$mmsg2."\n";
    if(count($imgatt) && count($imgatt_ext)==0){
      $mailmsg_res.="--".$boundary_parts."--\n\n";
    } else {
      $mailmsg_res.="--".$boundary_parts2."--\n\n";
    }

    if(count($imgatt)){
      foreach($imgatt as $img){
        $img=rtrim($img);
        $imgname="";
        if(preg_match("/^.+\/([^\/]+)\.([^\.]+)$/",$img,$tmp)){
          $imgname_base=$tmp[1];
          $imgname_ext=$tmp[2];
        }
        $tmpimgfile="/tmp/mail_".md5($img.filemtime($img));
        if(count($imgatt) && count($imgatt_ext)==0){
//          $mailmsg_res.="--".$boundary_parts."\n";
          $mailmsg_res.="--".$boundary."\n";
        } else {
          $mailmsg_res.="--".$boundary."\n";
        }
        $mailmsg_res.="Content-Type: ".guessMIMEType($imgname_base.".".$imgname_ext).";\n";
        $mailmsg_res.="\tname=\"".$imgname_base.".".$imgname_ext."\"\n";
        $mailmsg_res.="Content-Transfer-Encoding: base64\n";
//         if($att_inline){
          $mailmsg_res.="Content-ID: <".$imgname_base.">\n\n";
/*        } else {
          $mailmsg_res.="Content-Disposition: inline;\n";
          $mailmsg_res.="\tfilename=\"".$imgname_base.".".$imgname_ext."\"\n\n";
        }*/
        if(file_exists($tmpimgfile)){
          $pp = fopen($tmpimgfile,"r");
  //        echo "fopen"._nl;
        } else {
          $pp = popen("/usr/bin/mimencode -b \"".$img."\" > \"".$tmpimgfile."\" 2>/dev/null; cat \"".$tmpimgfile."\" ","r");
  //        echo "popen"._nl;
        }
        while(!feof($pp)) {
          $data = fread($pp,4096);
          $mailmsg_res.=$data;
        }
        pclose($pp);
      }
      if(count($imgatt) && count($imgatt_ext)==0){
  //      $mailmsg_res.="--".$boundary_parts."--\n";
        $mailmsg_res.="--".$boundary."--\n";
      } else {
        $mailmsg_res.="--".$boundary."--\n";
      }
    }

    if(count($imgatt_ext)){
      foreach($imgatt_ext as $img){
        $img=rtrim($img);
        $imgname="";
        if(preg_match("/^.+\/([^\/]+)\.([^\.]+)$/",$img,$tmp)){
          $imgname_base=$tmp[1];
          $imgname_ext=$tmp[2];
        }
        $tmpimgfile="/tmp/mail_".md5($img.filemtime($img));
        $mailmsg_res.="--".$boundary."\n";
        $mailmsg_res.="Content-Type: ".guessMIMEType($imgname_base.".".$imgname_ext).";\n";
        $mailmsg_res.="\tname=\"".$imgname_base.".".$imgname_ext."\"\n";
        $mailmsg_res.="Content-Transfer-Encoding: base64\n";
/*        if($att_inline){
          $mailmsg_res.="Content-ID: <".$imgname_base.">\n\n";
        } else {*/
          $mailmsg_res.="Content-Disposition: inline;\n";
          $mailmsg_res.="\tfilename=\"".$imgname_base.".".$imgname_ext."\"\n\n";
//         }
        if(file_exists($tmpimgfile)){
          $pp = fopen($tmpimgfile,"r");
  //        echo "fopen"._nl;
        } else {
          $pp = popen("/usr/bin/mimencode -b \"".$img."\" > \"".$tmpimgfile."\" 2>/dev/null; cat \"".$tmpimgfile."\" ","r");
  //        echo "popen"._nl;
        }
        while(!feof($pp)) {
          $data = fread($pp,4096);
          $mailmsg_res.=$data;
        }
        pclose($pp);
      }
      $mailmsg_res.="--".$boundary."--\n";
    }
    return(mail($mto,$mailsub,$mailmsg_res,$mailhead,"-B8BITMIME -f ".$mailfrom2));
  }
  // }}} sendmail3

?>
